''
'' A Freebasic Irrlicht Interface Test
'' F.Dodd (2006)
''
'' This is a pure test of creating a wrapper DLL for Irrlicht simplifying its
'' C++ architecture and providing a basic but functional 3D SDK for languages
'' that are not object oriented. At the moment this provides very few calls and
'' is merely a test for implementing such an interface.
''
'' This source uses Irrlicht definitions created by the Irrlicht authors
''

'' ////////////////////////////////////////////////////////////////////////////
'' Instructions to link the required DLL's
'#Inclib "IrrlichtWrapper"
#inclib "my_irr_wrapper"
'#Inclib "Irrlicht"

'' ////////////////////////////////////////////////////////////////////////////
'' Enumerated Type Definitions

' Rendering Device Types
ENUM IRR_DEVICE_TYPES
    IRR_EDT_NULL = 0            ' a NULL device with no display
    IRR_EDT_SOFTWARE            ' Irrlichts default software renderer
    IRR_EDT_SOFTWARE2           ' An improved quality software renderer
    IRR_EDT_OPENGL              ' hardware accelerated OpenGL renderer
    IRR_EDT_DIRECT3D8           ' hardware accelerated DirectX 8 renderer
    IRR_EDT_DIRECT3D9           ' hardware accelerated DirectX 9 renderer
END ENUM

' Material Flags
ENUM IRR_MATERIAL_FLAGS
    IRR_EMF_WIREFRAME = 0       ' render as wireframe outline
    IRR_EMF_GOURAUD_SHADING     ' render smoothly across polygons
    IRR_EMF_LIGHTING            ' material is effected by lighting
    IRR_EMF_ZBUFFER             ' enable z buffer
    IRR_EMF_ZWRITE_ENABLE       ' can write as well as read z buffer
    IRR_EMF_BACK_FACE_CULLING   ' cull polygons facing away
    IRR_EMF_BILINEAR_FILTER     ' enable bilinear filtering
    IRR_EMF_TRILINEAR_FILTER    ' enable trilinear filtering
    IRR_EMF_ANISOTROPIC_FILTER  ' reduce blur in distant textures
    IRR_EMF_FOG_ENABLE          ' enable fogging in the distance
    IRR_EMF_NORMALIZE_NORMALS   ' use when scaling dynamically lighted models
END ENUM

' Material Types
ENUM IRR_MATERIAL_TYPES
    IRR_EMT_SOLID = 0           ' Standard solid rendering uses one texture
    IRR_EMT_SOLID_2_LAYER       ' 2 blended textures using vertex alpha value
    IRR_EMT_LIGHTMAP            ' 2 textures: 0=color 1=lighting level and ignores vertex lighting
    IRR_EMT_LIGHTMAP_ADD        ' ... as above but adds levels instead of modulating between them
    IRR_EMT_LIGHTMAP_M2         ' ... as above but color levels are multiplied by 2 for brightening
    IRR_EMT_LIGHTMAP_M4         ' ... as above but color leels are multiplied by 4 for brightening
    IRR_EMT_LIGHTMAP_LIGHTING   ' 2 textures: 0=color 1=lighting level but supports dynamic lighting
    IRR_EMT_LIGHTMAP_LIGHTING_M2    ' ... as above but color levels are multiplied by 2 for brightening
    IRR_EMT_LIGHTMAP_LIGHTING_M4    ' ... as above but color leels are multiplied by 4 for brightening
    IRR_EMT_DETAIL_MAP          ' 2 blended textures: the first is a color map the second at a different scale adds and subtracts from the color to add detail
    IRR_EMT_SPHERE_MAP          ' makes the material appear reflective
    IRR_EMT_REFLECTION_2_LAYER  ' a reflective material blended with a color texture
    IRR_EMT_TRANSPARENT_ADD_COLOR   ' a transparency effect that simply adds a color texture to the background. the darker the color the more transparent it is.
    IRR_EMT_TRANSPARENT_ALPHA_CHANNEL   ' a transparency effect that uses the color textures alpha as a transparency level
    IRR_EMT_TRANSPARENT_ALPHA_CHANNEL_REF   ' a transparency effect that uses the color textures alpha, the pixel is only drawn if the alpha is > 127. this is a fast effect that does not blur edges and is ideal for leaves & grass etc.
    IRR_EMT_TRANSPARENT_VERTEX_ALPHA    ' a transparency effect that uses the vertex alpha value
    IRR_EMT_TRANSPARENT_REFLECTION_2_LAYER  ' a transparent & reflecting effect. the first texture is a reflection map, the second a color map. transparency is from vertex alpha
    IRR_EMT_NORMAL_MAP_SOLID    ' A solid normal map renderer. First texture is color, second is normal map. Only use nodes added with IrrAddStaticMeshForNormalMappingToScene. Only supports nearest two lights. Requires vertex and pixel shaders 1.1
    IRR_EMT_NORMAL_MAP_TRANSPARENT_ADD_COLOR    ' ... as above only with a transparency effect that simply adds the color to the background. the darker the color the more transparent it is.
    IRR_EMT_NORMAL_MAP_TRANSPARENT_VERTEX_ALPHA ' ... as above only with a transparency effect that uses the vertex alpha value
    IRR_EMT_PARALLAX_MAP_SOLID  ' similar to the solid normal map but more realistic providing virtual displacement of the surface. Uses the alpha channel of the normal map for height field displacement. Requires vertex shader 1.1 and pixel shader 1.4.
    IRR_EMT_PARALLAX_MAP_TRANSPARENT_ADD_COLOR  ' ... as above only with a transparency effect that simply adds the color to the background. the darker the color the more transparent it is.
    IRR_EMT_PARALLAX_MAP_TRANSPARENT_VERTEX_ALPHA   ' ... as above only wiht a transparency effect that uses the vertex alpha value
    IRR_EMT_ONE_TEXTURE_BLEND
    IRR_EMT_FOUR_DETAIL_MAP  ' 4 grayscale images in the channels of the first texture mixed with the vertex channels as alpha images
    IRR_EMT_TRANSPARENT_ADD_ALPHA_CHANNEL_REF ' as above only it adds the texture color rather than replacing it
    IRR_EMT_FORCE_32BIT = &h7fffffff
END ENUM

' Mouse events
ENUM IRR_MOUSE_EVENTS
    IRR_EMIE_LMOUSE_PRESSED_DOWN = 0
    IRR_EMIE_RMOUSE_PRESSED_DOWN
    IRR_EMIE_MMOUSE_PRESSED_DOWN
    IRR_EMIE_LMOUSE_LEFT_UP
    IRR_EMIE_RMOUSE_LEFT_UP
    IRR_EMIE_MMOUSE_LEFT_UP
    IRR_EMIE_MOUSE_MOVED
    IRR_EMIE_MOUSE_WHEEL
END ENUM

' MD2 Animation sequences
ENUM IRR_MD2_ANIM_SEQUENCES
    IRR_EMAT_STAND = 0
    IRR_EMAT_RUN
    IRR_EMAT_ATTACK
    IRR_EMAT_PAIN_A
    IRR_EMAT_PAIN_B
    IRR_EMAT_PAIN_C
    IRR_EMAT_JUMP
    IRR_EMAT_FLIP
    IRR_EMAT_SALUTE
    IRR_EMAT_FALLBACK
    IRR_EMAT_WAVE
    IRR_EMAT_POINT
    IRR_EMAT_CROUCH_STAND
    IRR_EMAT_CROUCH_WALK
    IRR_EMAT_CROUCH_ATTACK
    IRR_EMAT_CROUCH_PAIN
    IRR_EMAT_CROUCH_DEATH
    IRR_EMAT_DEATH_FALLBACK
    IRR_EMAT_DEATH_FALLFORWARD
    IRR_EMAT_DEATH_FALLBACKSLOW
    IRR_EMAT_BOOM
END ENUM

' For the FPS camera
Enum EKEY_ACTION
    EKA_MOVE_FORWARD = 0
    EKA_MOVE_BACKWARD
    EKA_STRAFE_LEFT
    EKA_STRAFE_RIGHT
    EKA_JUMP_UP
    EKA_COUNT
    EKA_FORCE_32BIT = &H7fffffff
End Enum

' Light types
Enum E_LIGHT_TYPE
    ELT_POINT = 0
    ELT_SPOT
    ELT_DIRECTIONAL
End Enum

' Joint animation modes
Enum IRR_JOINT_MODE
    IRR_JOINT_MODE_NONE = 0
    IRR_JOINT_MODE_READ
    IRR_JOINT_MODE_CONTROL
End Enum

' Pre-programmed shader constants
Enum IRR_SHADER_CONSTANTS
    IRR_NO_PRESET = 0
    IRR_INVERSE_WORLD
    IRR_WORLD_VIEW_PROJECTION
    IRR_CAMERA_POSITION
    IRR_TRANSPOSED_WORLD
End Enum

' Vertex shader program versions
Enum IRR_VERTEX_SHADER_VERSION
    EVST_VS_1_1 = 0
    EVST_VS_2_0
    EVST_VS_2_a
    EVST_VS_3_0
End Enum

' Pixel shader program versions
Enum IRR_PIXEL_SHADER_VERSION
    EPST_PS_1_1 = 0
    EPST_PS_1_2
    EPST_PS_1_3
    EPST_PS_1_4
    EPST_PS_2_0
    EPST_PS_2_a
    EPST_PS_2_b
    EPST_PS_3_0
End Enum

' enumeration for querying features of the video driver. 
Enum IRR_VIDEO_FEATURE_QUERY
    EVDF_RENDER_TO_TARGET = 0   ' Is driver able to render to a surface?  
    EVDF_HARDWARE_TL            ' Is hardeware transform and lighting supported?  
    EVDF_MULTITEXTURE           ' Are multiple textures per material possible?  
    EVDF_BILINEAR_FILTER        ' Is driver able to render with a bilinear filter applied?  
    EVDF_MIP_MAP                ' Can the driver handle mip maps?  
    EVDF_MIP_MAP_AUTO_UPDATE    ' Can the driver update mip maps automatically?  
    EVDF_STENCIL_BUFFER         ' Are stencilbuffers switched on and does the device support stencil buffers?  
    EVDF_VERTEX_SHADER_1_1      ' Is Vertex Shader 1.1 supported?  
    EVDF_VERTEX_SHADER_2_0      ' Is Vertex Shader 2.0 supported?  
    EVDF_VERTEX_SHADER_3_0      ' Is Vertex Shader 3.0 supported?  
    EVDF_PIXEL_SHADER_1_1       ' Is Pixel Shader 1.1 supported?  
    EVDF_PIXEL_SHADER_1_2       ' Is Pixel Shader 1.2 supported?  
    EVDF_PIXEL_SHADER_1_3       ' Is Pixel Shader 1.3 supported?  
    EVDF_PIXEL_SHADER_1_4       ' Is Pixel Shader 1.4 supported?  
    EVDF_PIXEL_SHADER_2_0       ' Is Pixel Shader 2.0 supported?  
    EVDF_PIXEL_SHADER_3_0       ' Is Pixel Shader 3.0 supported?  
    EVDF_ARB_VERTEX_PROGRAM_1   ' Are ARB vertex programs v1.0 supported?  
    EVDF_ARB_FRAGMENT_PROGRAM_1 ' Are ARB fragment programs v1.0 supported?  
    EVDF_ARB_GLSL               ' Is GLSL supported?  
    EVDF_HLSL                   ' Is HLSL supported?  
    EVDF_TEXTURE_NPOT           ' Are non-power-of-two textures supported?  
    EVDF_FRAMEBUFFER_OBJECT     ' Are framebuffer objects supported?
End Enum

' Enumeration flags defining the file format supported loading and saving
Enum IRR_MESH_FILE_FORMAT
    EMWT_IRR_MESH = 0            ' Irrlicht Native mesh writer, for static .irrmesh files.  
    EMWT_COLLADA                 ' COLLADA mesh writer for .dae and .xml files.  
    EMWT_STL                     ' STL mesh writer for .stl files.  
End Enum

' Enumeration flags defining the size of a patch in the terrain
Enum IRR_TERRAIN_PATCH_SIZE
    ETPS_9 = 9                    ' patch size of 9, at most, use 4 levels of detail with this patch size.  
    ETPS_17 = 17                  ' patch size of 17, at most, use 5 levels of detail with this patch size.  
    ETPS_33 = 33                  ' patch size of 33, at most, use 6 levels of detail with this patch size.  
    ETPS_65 = 65                  ' patch size of 65, at most, use 7 levels of detail with this patch size.  
    ETPS_129 = 129                ' patch size of 129, at most, use 8 levels of detail with this patch size. 
End Enum

' Enumeration flags telling the video driver in which format textures should be created. 
Enum IRR_TEXTURE_CREATION_FLAG
    ETCF_ALWAYS_16_BIT = 0      ' Forces the driver to create 16 bit textures always, independent of which format the file on disk has. When choosing this you may loose some color detail, but gain much speed and memory. 16 bit textures can be transferred twice as fast as 32 bit textures and only use half of the space in memory. When using this flag, it does not make sense to use the flags ETCF_ALWAYS_32_BIT, ETCF_OPTIMIZED_FOR_QUALITY, or ETCF_OPTIMIZED_FOR_SPEED at the same time.  
    ETCF_ALWAYS_32_BIT          ' Forces the driver to create 32 bit textures always, independent of which format the file on disk has. Please note that some drivers (like the software device) will ignore this, because they are only able to create and use 16 bit textures. When using this flag, it does not make sense to use the flags ETCF_ALWAYS_16_BIT, ETCF_OPTIMIZED_FOR_QUALITY, or ETCF_OPTIMIZED_FOR_SPEED at the same time.  
    ETCF_OPTIMIZED_FOR_QUALITY  ' Lets the driver decide in which format the textures are created and tries to make the textures look as good as possible. Usually it simply chooses the format in which the texture was stored on disk. When using this flag, it does not make sense to use the flags ETCF_ALWAYS_16_BIT, ETCF_ALWAYS_32_BIT, or ETCF_OPTIMIZED_FOR_SPEED at the same time.  
    ETCF_OPTIMIZED_FOR_SPEED    ' Lets the driver decide in which format the textures are created and tries to create them maximizing render speed. When using this flag, it does not make sense to use the flags ETCF_ALWAYS_16_BIT, ETCF_ALWAYS_32_BIT, or ETCF_OPTIMIZED_FOR_QUALITY, at the same time.  
    ETCF_CREATE_MIP_MAPS        ' Automatically creates mip map levels for the textures.  
    ETCF_NO_ALPHA_CHANNEL       ' Discard any alpha layer and use non-alpha color format.  
End Enum

' A color format specifies how color information is stored
Enum IRR_COLOR_FORMAT
    ECF_A1R5G5B5        ' 16 bit color format used by the software driver, and thus preferred by all other irrlicht engine video drivers. There are 5 bits for every color component, and a single bit is left for alpha information.  
    ECF_R5G6B5          ' Standard 16 bit color format.  
    ECF_R8G8B8          ' 24 bit color, no alpha channel, but 8 bit for red, green and blue.  
    ECF_A8R8G8B8        ' Default 32 bit color format. 8 bits are used for every component: red, green, blue and alpha.  
End Enum

Enum IRR_TEXTURE_BLEND
    BLEND_SCREEN = 0
    BLEND_ADD
    BLEND_SUBTRACT
    BLEND_MULTIPLY
    BLEND_DIVIDE
End Enum

Enum IRR_DEBUG
    EDS_OFF = 0
    EDS_BBOX = 1
    EDS_NORMALS = 2
    EDS_SKELETON = 4
    EDS_MESH_WIRE_OVERLAY = 8
    EDS_HALF_TRANSPARENCY = 16
    EDS_BBOX_BUFFERS = 32
    EDS_FULL = &hffffffff
End Enum

#Include "IrrlichtKeyEnum.bi"

'' ////////////////////////////////////////////////////////////////////////////
'' Constant Definitions

' No value, usually used when you do not wish to pass an object to a call
#define IRR_NO_OBJECT 0

' Parenting of nodes
#define IRR_NO_PARENT 0

' Flag definition
#define IRR_OFF 0
#define IRR_ON 1

' Input definitions
#define IRR_KEY_UP 0
#define IRR_KEY_DOWN 1

' The number of bits per pixel in the display
#define IRR_BITS_PER_PIXEL_16 16
#define IRR_BITS_PER_PIXEL_32 32

' System Device definitions
#define IRR_WINDOWED 0
#define IRR_FULLSCREEN 1

#define IRR_NO_SHADOWS 0
#define IRR_SHADOWS 1

#define IRR_IGNORE_EVENTS 0
#define IRR_CAPTURE_EVENTS 1

#define IRR_LINEAR_FOG 0
#define IRR_EXPONENTIAL_FOG 1

#define IRR_VERTICAL_SYNC_OFF 0
#define IRR_VERTICAL_SYNC_ON 1

' Filing system definitions
#define IRR_USE_CASE 0
#define IRR_IGNORE_CASE 1
#define IRR_USE_PATHS 0
#define IRR_IGNORE_PATHS 1

' 2D image definitions
#define IRR_IGNORE_ALPHA 0
#define IRR_USE_ALPHA 1

' Node definitions
#define IRR_INVISIBLE 0
#define IRR_VISIBLE 1

#define IRR_ONE_SHOT 0
#define IRR_LOOP 1

' Particle definitions
#define IRR_NO_EMITTER 0
#define IRR_DEFAULT_EMITTER 1

#define IRR_ATTRACT 1
#define IRR_REPEL 0

' GUI Interface definitions
#define IRR_GUI_NO_BORDER 0
#define IRR_GUI_BORDER 1
#define IRR_GUI_NO_WRAP 0
#define IRR_GUI_WRAP 1

' Spherical Terrain Faces
#define IRR_TOP_FACE 0
#define IRR_FRONT_FACE 1
#define IRR_BACK_FACE 2
#define IRR_LEFT_FACE 3
#define IRR_RIGHT_FACE 4
#define IRR_BOTTOM_FACE 5


'' ////////////////////////////////////////////////////////////////////////////
'' Type Definitions

' Irrlicht Keyboard and Mouse Event Structures
TYPE IRR_MOUSE_EVENT
    action as uinteger
    delta as single
    x as integer
    y as integer
END TYPE

TYPE IRR_KEY_EVENT
    key as uinteger
    direction as uinteger
    flags as uinteger
END TYPE

' Particle emitter properties
TYPE IRR_PARTICLE_EMITTER
    min_box_x as single     ' The bounding box for the emitter
    min_box_y as single
    min_box_z as single

    max_box_x as single
    max_box_y as single
    max_box_z as single

    direction_x as single
    direction_y as single
    direction_z as single

    min_paritlcles_per_second as uinteger
    max_paritlcles_per_second as uinteger

    min_start_color_red as integer
    min_start_color_green as integer
    min_start_color_blue as integer

    max_start_color_red as integer
    max_start_color_green as integer
    max_start_color_blue as integer
    
    min_lifetime as uinteger
    max_lifetime as uinteger
    
    max_angle_degrees as integer
END TYPE

' a vertex used in creating custom static mesh objects
TYPE IRR_VERT
    x as single             ' The x position of the vertex
    y as single             ' The y position of the vertex
    z as single             ' The z position of the vertex
    normal_x as single      ' The x normal of the vertex
    normal_y as single      ' The y normal of the vertex
    normal_z as single      ' The z normal of the vertex
    vcolor as uinteger      ' The 32bit ARGB color of the vertex
    texture_x as single     ' the x co-ordinate of the vertex on the texture (0 to 1)
    texture_y as single     ' the y co-ordinate of the vertex on the texture (0 to 1)
END TYPE

' a vector consisting of 3 float values
TYPE IRR_VECTOR
    x as single
    y as single
    z as single
END TYPE

' the contents of a key event
Type SKeyMap
    Action As EKEY_ACTION
    KeyCode As EKEY_CODE
End Type

' a material representing a new shader
Type IRR_SHADER
    ' use the material type in a call to IrrSetNodeMaterialType
    material_type as INTEGER
    ' the following properties are reserved for internal use changing them can
    ' result in crashes and memory leakage
    material_object as UINTEGER PTR
    next_shader as UINTEGER PTR
End Type


' .............................................................................
' Irrlicht Object Types - these are used to represent objects in the Irrlicht
' engine and their purpose is to warn the programmer of type errors
' (thanks to v1ctor for the advice on this correction)
TYPE irr_mesh as UINTEGER PTR

TYPE irr_image as UINTEGER PTR

TYPE irr_texture as UINTEGER PTR

TYPE irr_material as UINTEGER PTR

TYPE irr_node as UINTEGER PTR

TYPE irr_camera as UINTEGER PTR

TYPE irr_terrain as UINTEGER PTR

TYPE irr_selector as UINTEGER PTR

TYPE irr_particle_system as UINTEGER PTR

TYPE irr_emitter as UINTEGER PTR

TYPE irr_affector as UINTEGER PTR

TYPE irr_animator as UINTEGER PTR

TYPE irr_font as UINTEGER PTR

Type irr_scene_node As uInteger Ptr

UNION irr_model
    node as irr_node
    camera as irr_camera
    terrain as irr_terrain
    particles as irr_particle_system
END UNION



'' ////////////////////////////////////////////////////////////////////////////
'' System Functions

' Start the Irrlicht interface
Declare Sub IrrStart CDECL Alias "IrrStart" ( ByVal device_type As IRR_DEVICE_TYPES, ByVal iwidth As Integer, ByVal iheight As Integer, ByVal bitsperpixel As unSigned Integer, ByVal fullscreen As Integer, ByVal use_shadows As Integer, ByVal iCaptureMouse As Integer, ByVal vsync as Integer = IRR_OFF )

' allow transparency to write to the z buffer
Declare Sub IrrTransparentZWrite CDECL Alias "IrrTransparentZWrite" ()

' determine if the Irrlicht viewport is running. 0 = No 1 = Yes
declare function IrrRunning CDECL alias "IrrRunning" () as integer

' initialise the frame drawing cycle, erasing the canvas ready for drawing
declare sub IrrBeginScene CDECL alias "IrrBeginScene" ( byval red as integer, byval green as integer, byval blue as integer)

' draw the scene to the canvas
declare sub IrrDrawScene CDECL alias "IrrDrawScene" ()

' draw scene manager objects to a texture surface, the texture must have been
' created with a call to IrrCreateRenderTargetTexture
declare sub IrrDrawSceneToTexture CDECL alias "IrrDrawSceneToTexture" ( byval texture as irr_texture )

' draw the user interface to the canvas
declare sub IrrDrawGUI CDECL alias "IrrDrawGUI" ()

' end all drawing operations and display the rendered frame
declare sub IrrEndScene CDECL alias "IrrEndScene" ()

' Stop the Irrlicht engine freeing up its resources
declare sub IrrStop CDECL alias "IrrStop" ()

' Query a feature of the video card, returns 1 if it is supported and 0 if it isnt
declare function IrrQueryFeature CDECL alias "IrrQueryFeature" ( byval Feature as IRR_VIDEO_FEATURE_QUERY ) as uinteger

' get the current time in milliseconds
declare function IrrGetTime CDECL alias "IrrGetTime" () as uinteger

' get the current frame rate
declare function IrrGetFPS CDECL alias "IrrGetFPS" () as integer

' get the number of primitives (mostly triangles) drawn in the last frame
declare function IrrGetPrimitivesDrawn CDECL alias "IrrGetPrimitivesDrawn" () as uinteger


' set the caption in the irrlicht window
declare sub IrrSetWindowCaption CDECL alias "IrrSetWindowCaption" ( byval text as wstring ptr )

' make 32 bit representation of an Alpha, Red, Green, Blue color
function IrrMakeARGB ( Alpha as integer, Red as integer, Green as integer, Blue as integer ) as uinteger
    IrrMakeARGB = (Alpha SHL 24) + (Red SHL 16) + (Green SHL 8) + Blue
end function



'' ////////////////////////////////////////////////////////////////////////////
'' Input Event Functions

' find out if there is a key event ready to be read. returns 1 if there is an
' event available (the event receiver must have been started when the system
' was initialised)
declare function IrrKeyEventAvailable CDECL alias "IrrKeyEventAvailable" () as integer

' read a key event out
declare function IrrReadKeyEvent CDECL alias "IrrReadKeyEvent" () as IRR_KEY_EVENT PTR

' find out if there is a mouse event ready to be read. returns 1 if there is an
' event available (the event receiver must have been started when the system
' was initialised)
declare function IrrMouseEventAvailable CDECL alias "IrrMouseEventAvailable" () as integer

' read a mouse event out
declare function IrrReadMouseEvent CDECL alias "IrrReadMouseEvent" () as IRR_MOUSE_EVENT PTR

' show and hide the mouse pointer
#define IrrHideMouse IrrDisplayMouse(0)
#define IrrShowMouse IrrDisplayMouse(1)
declare sub IrrDisplayMouse CDECL alias "IrrDisplayMouse" ( byval show_mouse as uinteger)

' set the position of the mouse pointer and return the relative change in position
declare sub IrrSetMousePosition CDECL alias "IrrSetMousePosition" ( byref x as single, byref y as single )


'' ////////////////////////////////////////////////////////////////////////////
'' Filing System Functions

' add zip archive to the filing system allowing you to load files straight out
' of the zip file. common pk3 files are simply zip files
declare sub IrrAddZipFile CDECL alias "IrrAddZipFile" ( byval path as zstring ptr, byval ignore_case as uinteger, byval ignore_paths as uinteger)

' set irrlicht current working directory
declare sub IrrChangeWorkingDirectory CDECL alias "IrrChangeWorkingDirectory" ( byval path as zstring ptr )

' get irrlicht current working directory
declare function IrrGetWorkingDirectory CDECL alias "IrrGetWorkingDirectory" () as zstring ptr



'' ////////////////////////////////////////////////////////////////////////////
'' 2D Functions

' sets texture creation flags, use IRR_ON and IRR_OFF to set the flag value
declare sub IrrSetTextureCreationFlag CDECL alias "IrrSetTextureCreationFlag" ( byval flag as IRR_TEXTURE_CREATION_FLAG, byval value as uinteger)

' load an image (images do not use video memory and cannot texture objects
declare function IrrGetImage CDECL alias "IrrGetImage" ( byval path as zstring ptr ) as irr_image

' load a texture object
declare function IrrGetTexture CDECL alias "IrrGetTexture" ( byval path as zstring ptr ) as irr_texture

' create a blank texture
declare function IrrCreateTexture CDECL alias "IrrCreateTexture" ( byval path as zstring ptr, byval x as integer, byval y as integer, byval format as IRR_COLOR_FORMAT ) as irr_texture

' create a blank image
declare function IrrCreateImage CDECL alias "IrrCreateImage" ( byval x as integer, byval y as integer, byval format as IRR_COLOR_FORMAT ) as irr_image

' create a texture that is suitable for the scene manager to use as a surface to
' which it can render its 3d object. each the dimentions must be a power of two
' for example 128x128 or 256x256
declare function IrrCreateRenderTargetTexture CDECL alias "IrrCreateRenderTargetTexture" ( byval x as integer, byval y as integer ) as irr_texture

' remove a texture freeing its resources
declare sub IrrRemoveTexture CDECL alias "IrrRemoveTexture" ( byval texture as irr_texture )

' create a normal map from a grayscale heightmap texture
declare sub IrrMakeNormalMapTexture CDECL alias "IrrMakeNormalMapTexture" ( byval texture as irr_texture, byval amplitude as single)

' locks the texture and returns a pointer to the pixels
declare function IrrLockTexture CDECL alias "IrrLockTexture" ( byval texture as irr_texture ) as uinteger ptr

' unlock the texture, presumably after it has been modified and recreate the mipmap levels
declare sub IrrUnlockTexture CDECL alias "IrrUnlockTexture" ( byval texture as irr_texture )

' locks the image and returns a pointer to the pixels
declare function IrrLockImage CDECL alias "IrrLockImage" ( byval image as irr_image ) as uinteger ptr

' unlock the image, presumably after it has been modified
declare sub IrrUnlockImage CDECL alias "IrrUnlockImage" ( byval image as irr_image )

' blend the source texture into the destination texture to create a single texture
declare function IrrBlendTextures CDECL alias "IrrBlendTextures" ( byval destination_texture as irr_texture, byval source_texture as irr_texture, byval x_offset as integer, byval y_offset as integer, byval operation as integer ) as integer

' color key the image making parts of it transparent
declare sub IrrColorKeyTexture CDECL alias "IrrColorKeyTexture" ( byval texture as irr_texture, byval red as integer, byval green as integer, byval blue as integer)

' draw a 2d image to the display
declare sub IrrDraw2DImage CDECL alias "IrrDraw2DImage" ( byval texture as irr_texture, byval x as integer, byval y as integer )

' draw an image to the screen from a specified rectangle in the source image
' this enables you to place lots of images into one texture for 2D drawing. in
' addition this function also uses image alpha channels
declare sub IrrDraw2DImageElement CDECL alias "IrrDraw2DImageElement" ( byval texture as irr_texture, byval x as integer, byval y as integer, byval source_tx as integer, byval source_ty as integer, byval source_bx as integer, byval source_by as integer, byval usealpha as integer )

' load a bitmap based font
declare function IrrGetFont CDECL alias "IrrGetFont" ( byval path as zstring ptr ) as irr_font

' draw text to the display using a bitmap font
declare sub Irr2DFontDraw CDECL alias "Irr2DFontDraw" ( byval font as irr_font, byval text as wstring ptr, byval top_x as integer, byval top_y as integer, byval bottom_x as integer, byval bottom_y as integer )

' save a screenshot out to a file
declare sub IrrSaveScreenShot CDECL alias "IrrSaveScreenShot" ( byval text as zstring ptr )



'' ////////////////////////////////////////////////////////////////////////////
'' Material and GPU Programming Functions

' Set how shiny the material is the higher the value the more defined the highlights
declare sub IrrMaterialSetShininess CDECL alias "IrrMaterialSetShininess" ( byval material as irr_material, byval shininess as single )

' The color of specular highlights on the object
declare sub IrrMaterialSetSpecularColor CDECL alias "IrrMaterialSetSpecularColor" ( byval material as irr_material, byval Alpha as uinteger, byval Red as uinteger, byval Green as uinteger, byval Blue as uinteger )

' The color of diffuse lighting on the object
declare sub IrrMaterialSetDiffuseColor CDECL alias "IrrMaterialSetDiffuseColor" ( byval material as irr_material, byval Alpha as uinteger, byval Red as uinteger, byval Green as uinteger, byval Blue as uinteger )

' The color of ambient light reflected by the object
declare sub IrrMaterialSetAmbientColor CDECL alias "IrrMaterialSetAmbientColor" ( byval material as irr_material, byval Alpha as uinteger, byval Red as uinteger, byval Green as uinteger, byval Blue as uinteger )

' The color of light emitted by the object
declare sub IrrMaterialSetEmissiveColor CDECL alias "IrrMaterialSetEmissiveColor" ( byval material as irr_material, byval Alpha as uinteger, byval Red as uinteger, byval Green as uinteger, byval Blue as uinteger )

' set material specific parameter
declare sub IrrMaterialSetMaterialTypeParam CDECL alias "IrrMaterialSetMaterialTypeParam" ( byval material as irr_material, byval value as single )


' Creates a Vertex shader constant that allows you to change the value of a
' constant inside a GLSL shader during the execution of the program simply
' assign one of the preset constants to the constant name or attach the constant
' to an array of floats and change the constant simply by changing the values
' in your array
' Returns: 1 if the constant was sucessfully created
declare function IrrCreateNamedVertexShaderConstant CDECL alias "IrrCreateNamedVertexShaderConstant" ( byval shader as IRR_SHADER ptr, byval const_name as zstring ptr, byval const_preset as integer, byval const_data as single ptr, byval data_count as integer ) as integer

' Creates a Pixel shader constant, the operation of the function is the same
' as that of the vertex shader constant above.
' Returns: 1 if a constant was sucessfully created
declare function IrrCreateNamedPixelShaderConstant CDECL alias "IrrCreateNamedPixelShaderConstant" ( byval shader as IRR_SHADER ptr, byval const_name as zstring ptr, byval const_preset as integer, byval const_data as single ptr, byval data_count as integer ) as integer

' the same as IrrCreateNamedVertexShaderConstant but uses an address instead
' of a name for low level shaders
' Returns: 1 if a constant was sucessfully created
declare function IrrCreateAddressedVertexShaderConstant CDECL alias "IrrCreateAddressedVertexShaderConstant" ( byval shader as IRR_SHADER ptr, byval const_address as integer, byval const_preset as integer, byval const_data as single ptr, byval data_count as integer ) as integer

' the same as IrrCreateNamedPixelShaderConstant but uses an address instead
' of a name for low level shaders
' Returns: 1 if a constant was sucessfully created
declare function IrrCreateAddressedPixelShaderConstant CDECL alias "IrrCreateAddressedPixelShaderConstant" ( byval shader as IRR_SHADER ptr, byval const_address as integer, byval const_preset as integer, byval const_data as single ptr, byval data_count as integer ) as integer

' creates a new material using a high level shading language. these programs
' that run on the graphics card can greatly add to the realism of the output
' but only modern cards will support them. Currently only the HLSL/D3D9 and
' GLSL/OpenGL is supported.
' 
' vertex program:
'	String containing the source of the vertex shader program. This can be 0 if
'	no vertex program shall be used.  
' vertex_start_function:
'	Name of the entry function of the vertex shader program
' vertex_program_type:
'	Vertex shader version used to compile the GPU program
' pixel_program:
'	String containing the source of the pixel shader program. This can be 0 if
'	no pixel shader shall be used.  
' pixel_start_function:
'	Entry name of the function of the pixel shader program
' pixel_program_type:
'	Pixel shader version used to compile the GPU program
' baseMaterial:
'	Base material which renderstates will be used to shade the material.  
'
' Return:
'   Returns a type that contains a material_type number that can be used to
' shade nodes with this new material. If the shader could not be created it
' will return 0
'
declare function IrrAddHighLevelShaderMaterial CDECL alias "IrrAddHighLevelShaderMaterial" ( _
        byval vertex_program as zstring ptr, _
        byval vertex_start_function as zstring ptr, _
        byval vertex_prog_type as uinteger, _
        byval pixel_program as zstring ptr, _
        byval pixel_start_function as zstring ptr, _
        byval pixel_prog_type as uinteger, _
        byval material_type as uinteger ) as IRR_SHADER ptr

' Identical to IrrAddHighLevelShaderMaterial only this varient attempts to load
' the GPU programs from files
declare function IrrAddHighLevelShaderMaterialFromFiles CDECL alias "IrrAddHighLevelShaderMaterialFromFiles" ( _
        byval vertex_program_filename as zstring ptr, _
        byval vertex_start_function as zstring ptr, _
        byval vertex_prog_type as uinteger, _
        byval pixel_program_filename as zstring ptr, _
        byval pixel_start_function as zstring ptr, _
        byval pixel_prog_type as uinteger, _
        byval material_type as uinteger ) as IRR_SHADER ptr

' creates a new material using a high level shading language. these programs
' that run on the graphics card can greatly add to the realism of the output
' but only modern cards will support them. Currently only the HLSL/D3D9 and
' GLSL/OpenGL is supported.
' 
' vertex program:
'	String containing the source of the vertex shader program. This can be 0 if
'	no vertex program shall be used. For DX8 programs, the will always input
'	registers look like this: v0: position, v1: normal, v2: color, v3: texture
'	cooridnates, v4: texture coordinates 2 if available. For DX9 programs, you
'	can manually set the registers using the dcl_ statements.  
' pixel_program:
'	String containing the source of the pixel shader program. This can be 0 if
'	no pixel shader shall be used.  
' pixel_start_function:
'	Entry name of the function of the pixel shader program
' pixel_program_type:
'	Pixel shader version used to compile the GPU program
' baseMaterial:
'	Base material which renderstates will be used to shade the material.  
'
' Return:
'   Returns a type that contains a material_type number that can be used to
' shade nodes with this new material. If the shader could not be created it
' will return 0
'
declare function IrrAddShaderMaterial CDECL alias "IrrAddShaderMaterial" ( _
        byval vertex_program as zstring ptr, _
        byval pixel_program as zstring ptr, _
        byval material_type as uinteger ) as IRR_SHADER ptr

' Identical to IrraddShaderMaterial only this varient attempts to load the GPU
' programs from file.
declare function IrrAddShaderMaterialFromFiles CDECL alias "IrrAddShaderMaterialFromFiles" ( _
        byval vertex_program_filename as zstring ptr, _
        byval pixel_program_filename as zstring ptr, _
        byval material_type as uinteger ) as IRR_SHADER ptr



'' ////////////////////////////////////////////////////////////////////////////
'' Scene Manager Functions

' load a mesh from a file
declare function IrrGetMesh CDECL alias "IrrGetMesh" ( byval path as zstring ptr ) as irr_mesh

' create a mesh from an array of verticies, an array of incidies to those
' verticies that connect them to form triangles and finally texture co-ordinates
' to map the verticies against a texture plane
declare function IrrCreateMesh CDECL alias "IrrCreateMesh" (byval mesh_name as zstring ptr, byval vertex_count as integer, byref vertices as IRR_VERT, byval indices_count as integer, byref indices as ushort) as irr_mesh

' adds a hill plane mesh to the mesh pool
declare Function IrrAddHillPlaneMesh CDECL Alias "IrrAddHillPlaneMesh" ( ByVal mesh_name As zString Ptr, ByVal tileSizeX As Single, ByVal tileSizeY As Single, ByVal tileCountX As Integer, ByVal tileCountY As Integer, ByVal material As uInteger Ptr = 0, ByVal hillHeight As Single = 0, ByVal countHillsX As Single = 0, ByVal countHillsY As Single = 0, ByVal textureRepeatCountX As Single = 1, ByVal textureRepeatCountY As Single = 1 ) as irr_mesh

' write the first frame of the supplied animated mesh out to a file using the specified file format
declare function IrrWriteMesh CDECL alias "IrrWriteMesh" ( byval mesh as irr_mesh, byval file_format as IRR_MESH_FILE_FORMAT, byval path as zstring ptr ) as uinteger

' remove a loaded mesh from the scene cache useful if you are dealing with large
' numbers of meshes that you dont want cached in memory at the same time (for
' example when swapping between BSP maps for levels
declare sub IrrRemoveMesh CDECL alias "IrrRemoveMesh" (byval mesh as irr_mesh)

' Clears all meshes that are held in the mesh cache but not used anywhere else.
declare sub IrrClearUnusedMeshes CDECL alias "IrrClearUnusedMeshes" ()

' get the number of indicies in the mesh buffer
declare function IrrGetMeshIndexCount CDECL alias "IrrGetMeshIndexCount" ( byval mesh as irr_mesh, byval frame as integer ) as integer

' copy the indicies of a mesh into the supplied buffer, the caller must ensure
' that the buffer is big enough to store the data
declare sub IrrGetMeshIndices CDECL alias "IrrGetMeshIndices" ( byval mesh as irr_mesh, byval frame as integer, byref indicies as ushort )

' copy the indicies in the supplied buffer into the mesh , the caller must
' ensure that the buffer is big enough to supply all of the data
declare sub IrrSetMeshIndices CDECL alias "IrrSetMeshIndices" ( byval mesh as irr_mesh, byval frame as integer, byref indicies as ushort )

' get the number of vertices in the mesh buffer
declare function IrrGetMeshVertexCount CDECL alias "IrrGetMeshVertexCount" ( byval mesh as irr_mesh, byval frame as integer ) as integer

' copy the vertices of a mesh into the supplied buffer, the caller must ensure
' that the buffer is big enough to store the data
declare sub IrrGetMeshVertices CDECL alias "IrrGetMeshVertices" ( byval mesh as irr_mesh, byval frame as integer, byref verticies as IRR_VERT )

' copy the vertices in the supplied buffer into the mesh , the caller must
' ensure that the buffer is big enough to supply all of the data
declare sub IrrSetMeshVertices CDECL alias "IrrSetMeshVertices" ( byval mesh as irr_mesh, byval frame as integer, byref verticies as IRR_VERT )

' gets the scenes root node, all scene nodes are children of this node
declare function IrrGetRootSceneNode CDECL alias "IrrGetRootSceneNode" () as irr_node

' add a loaded mesh to the scene as a new node
declare function IrrAddMeshToScene CDECL alias "IrrAddMeshToScene" ( byval mesh as irr_mesh) as irr_node

' add a loaded mesh to the scene as an octtree. this is an efficient method
' for large complex meshes like BSP maps that occlude themselves a lot
declare function IrrAddMeshToSceneAsOcttree CDECL alias "IrrAddMeshToSceneAsOcttree" (byval mesh as irr_mesh) as irr_node

' add a loaded mesh to the irrlicht scene manager using a static mesh buffer,
' tangent vertex information is added to allow the scene node to be used with
' normal and parallax mapping materials
declare function IrrAddStaticMeshForNormalMappingToScene CDECL alias "IrrAddStaticMeshForNormalMappingToScene" ( byval mesh as irr_mesh) as irr_node


' load a scene created by irrEdit
declare Sub IrrLoadScene CDECL Alias "IrrLoadScene" (ByVal filename As zstring ptr) 

' save the current scene into a file that can be loaded by irrEdit
declare Sub IrrSaveScene CDECL Alias "IrrSaveScene" (ByVal filename As zstring ptr) 

' get a scene node based on its ID and returns null if no node is found
declare function IrrGetSceneNodeFromId CDECL alias "IrrGetSceneNodeFromId" ( byval id as integer) as irr_node

' get a node in the scene based upon its name and returns null is no node is found
declare function IrrGetSceneNodeFromName CDECL alias "IrrGetSceneNodeFromName" ( byval name as zstring ptr) as irr_node


' adds a billboard to the scene this is a flat 3D textured sprite
declare function IrrAddBillBoardToScene CDECL alias "IrrAddBillBoardToScene" ( byval sizex as single, byval sizey as single, byval x as single = 0, byval y as single = 0, byval z as single = 0 ) as irr_node

' add a particle system to the irrlicht scene manager
Declare Function IrrAddParticleSystemToScene CDECL Alias "IrrAddParticleSystemToScene" ( ByVal add_emitter As Integer, ByVal parent As irr_node = 0, ByVal id As Integer = -1, ByVal posX As Single = 0, ByVal posY As Single = 0, ByVal posZ As Single = 0, ByVal rotX As Single = 0, ByVal rotY As Single = 0, ByVal rotZ As Single = 0, ByVal scaleX As Single = 1, ByVal scaleY As Single = 1, ByVal scaleZ As Single = 1) As irr_particle_system

' add a skybox to the irrlicht scene manager based on six pictures
declare function IrrAddSkyBoxToScene CDECL alias "IrrAddSkyBoxToScene" ( byval up_texture as irr_texture, byval down_texture as irr_texture, byval left_texture as irr_texture, byval right_texture as irr_texture, byval front_texture as irr_texture, byval back_texture as irr_texture ) as irr_node

' add a skydome to the irrlicht scene manager
declare Function IrrAddSkyDomeToScene CDECL Alias "IrrAddSkyDomeToScene" ( ByVal the_texture As irr_texture, ByVal horizontal_res As uInteger, ByVal vertical_res As uInteger, ByVal texture_percentage As Double, ByVal sphere_percentage As Double, ByVal radius as Double = 1000.0 ) As irr_node 

' adds a test node to the scene, the node is a simple cube
declare function IrrAddTestSceneNode CDECL alias "IrrAddTestSceneNode" () as irr_node

' adds a simple cube node to the scene
declare function IrrAddCubeSceneNode CDECL alias "IrrAddCubeSceneNode" ( byval size as single ) as irr_node

' adds a simple sphere node to the scene
declare function IrrAddSphereSceneNode CDECL alias "IrrAddSphereSceneNode" ( ByVal redius as single, ByVal polyCount as integer ) as irr_node

' adds a simple sphere node to the scene
declare function IrrAddSphereSceneMesh CDECL alias "IrrAddSphereSceneMesh" ( byval name as zstring ptr, ByVal redius as single, ByVal polyCount as integer ) as irr_mesh

' add a scene node for rendering an animated water surface mesh
declare Function IrrAddWaterSurfaceSceneNode CDECL Alias "IrrAddWaterSurfaceSceneNode" ( ByVal mesh As irr_mesh, ByVal waveHeight As Single = 2.0, ByVal waveSpeed As Single = 300.0, ByVal waveLength As Single = 10.0, ByVal parent As irr_scene_node = 0, ByVal id As Integer = -1, ByVal positionX As Single = 0, ByVal positionY As Single = 0, ByVal positionZ As Single = 0, ByVal rotationX As Single = 0, ByVal rotationY As Single = 0, ByVal rotationZ As Single = 0, ByVal scaleX As Single = 1.0, ByVal scaleY As Single = 1.0, ByVal scaleZ As Single = 1.0) as irr_node

' add a scene node for rendering an animated water surface mesh
declare Function IrrAddClouds CDECL Alias "IrrAddClouds" (byval texture as irr_texture, byval lod as uinteger, byval depth as uinteger, byval density as uinteger ) as irr_node

' add a scene node for rendering a lens flare effect
declare Function IrrAddLensFlare CDECL Alias "IrrAddLensFlare" (byval texture as irr_texture) as irr_node

' add a scene node for rendering a patch of grass
declare Function IrrAddGrass CDECL Alias "IrrAddGrass" ( _
        byval terrain as irr_terrain, _
        byval x as integer, _
        byval y as integer, _
        byval patchSize as integer, _
        byval grassScale as single, _
        byval maxDensity as uinteger, _
        byval dataPositionX as integer, _
        byval dataPositionY as integer, _
        byval heightMap as irr_image, _
        byval textureMap as irr_image, _
        byval grassMap as irr_image, _
        byval grassTexture as irr_texture ) as irr_node

' adds a zonemanagement object
declare function IrrAddZoneManager CDECL alias "IrrAddZoneManager" ( _
        byval nearDistance as single = 0, _
        byval farDistance as single = 12000 ) as irr_node

declare sub IrrSetZoneManagerProperties CDECL alias "IrrSetZoneManagerProperties" ( _
        byval zoneNode as irr_node, _
        byval nearDistance as single, _
        byval farDistance as single, _
        byval accumulateChildBoxes as uinteger )

declare sub IrrSetZoneManagerBoundingBox CDECL alias "IrrSetZoneManagerBoundingBox" ( _
        byval zoneNode as irr_node, _
        byval x as single, _
        byval y as single, _
        byval z as single, _
        byval boxWidth as single, _
        byval boxHeight as single, _
        byval boxDepth as single )

declare sub IrrSetZoneManagerAttachTerrain CDECL alias "IrrSetZoneManagerAttachTerrain" ( _
        byval zoneNode as irr_node, _
        byval terrain as irr_terrain, _
        byval structureMapFile as zstring ptr, _
        byval colorMapFile as zstring ptr, _
        byval detailMapFile as zstring ptr, _
        byval ImageX as integer, _
        byval ImageY as integer, _
        byval sliceSize as integer )

' set grass density
declare sub IrrSetGrassDensity CDECL alias "IrrSetGrassDensity" ( byval grass as irr_node, byval density as integer, byval distance as single )

' set wind effect on grass
declare sub IrrSetGrassWind CDECL alias "IrrSetGrassWind" ( byval grass as irr_node, byval strength as single, byval resolution as single )

declare function IrrGetGrassDrawCount CDECL alias "IrrGetGrassDrawCount" ( byval grass as irr_node ) as uinteger

declare sub IrrSetFlareScale CDECL alias "IrrSetFlareScale" ( byval flare as irr_node, byval source as single, byval optics as single )

' set the color of shadows in the scene
declare sub IrrSetShadowColor CDECL alias "IrrSetShadowColor" ( byval Alpha as integer, byval R as integer, byval G as integer, byval B as integer )

' set the scene fog
declare sub IrrSetFog CDECL alias "IrrSetFog" ( byval R as integer, byval G as integer, byval B as integer, byval fogtype as integer, byval fog_start as single, byval fog_end as single, byval density as single )

' draw a 3D line into the display
declare sub IrrDraw3DLine CDECL alias "IrrDraw3DLine" ( byval xStart as single, byval yStart as single, byval zStart as single, byval xEnd as single, byval yEnd as single, byval zEnd as single, byval R as uinteger, byval G as uinteger, byval B as uinteger )

declare sub IrrSetSkyDomeColor CDECL alias "IrrSetSkyDomeColor" ( _
        byval dome as irr_node, _
        byval horizontalRed as uinteger, _
        byval horizontalGreen as uinteger, _
        byval horizontalBlue as uinteger, _
        byval zenithRed as uinteger, _
        byval zenithGreen as uinteger, _
        byval zenithBlue as uinteger )

' apply a colored band to the skydome this can be used to represent a change in
' lighting at the horizon
declare sub IrrSetSkyDomeColorBand CDECL alias "IrrSetSkyDomeColorBand" ( _
        byval dome as irr_node, _
        byval horizontalRed as uinteger, _
        byval horizontalGreen as uinteger, _
        byval horizontalBlue as uinteger, _
        byval bandVerticalPosition as integer, _
        byval bandFade as single, _
        byval addative as uinteger )

declare sub IrrSetSkyDomeColorPoint CDECL alias "IrrSetSkyDomeColorPoint" ( _
        byval dome as irr_node, _
        byval Red as uinteger, _
        byval Green as uinteger, _
        byval Blue as uinteger, _
        byval pointXPosition as single, _
        byval pointYPosition as single, _
        byval pointZPosition as single, _
        byval pointRadius as single, _
        byval pointFade as single, _
        byval addative as uinteger )



'' ////////////////////////////////////////////////////////////////////////////
'' Scene Node Functions

' Get the number of materials associated with the node
declare function IrrGetMaterialCount CDECL alias "IrrGetMaterialCount" ( byval node as irr_node) as uinteger

' Get the material associated with the node at the particular index
declare function IrrGetMaterial CDECL alias "IrrGetMaterial" ( byval node as irr_node, byval index as uinteger) as irr_material

' apply an image file as a texturing material to a node
declare sub IrrSetNodeMaterialTexture CDECL alias "IrrSetNodeMaterialTexture" ( byval node as irr_node, byval texture as irr_texture, byval material_index as integer )

' set a material flag property on a node. the value can be either 0 or 1
declare sub IrrSetNodeMaterialFlag CDECL alias "IrrSetNodeMaterialFlag" ( byval node as irr_node, byval flag as IRR_MATERIAL_FLAGS, byval value as uinteger )

' set the way the materials applied to the node are rendered
declare sub IrrSetNodeMaterialType CDECL alias "IrrSetNodeMaterialType" ( byval node as irr_node, byval mat_type as IRR_MATERIAL_TYPES )


' move a node in the scene to a new position
declare sub IrrSetNodePosition CDECL alias "IrrSetNodePosition" ( byval node as irr_node, byval X as single, byval Y as single, byval Z as single )

' rotate a node in the scene
declare sub IrrSetNodeRotation CDECL alias "IrrSetNodeRotation" ( byval node as irr_node, byval X as single, byval Y as single, byval Z as single )

' scale a node in the scene
declare sub IrrSetNodeScale CDECL alias "IrrSetNodeScale" ( byval node as irr_node, byval X as single, byval Y as single, byval Z as single )

'render debugging information for nodes to the display
declare sub IrrDebugDataVisible CDECL alias "IrrDebugDataVisible" ( byval node as irr_node, byval visible as integer = &hFFFFFFFF )


' get the position of the node in the scene
declare sub IrrGetNodePosition CDECL alias "IrrGetNodePosition" ( byval node as irr_node, byref X as single, byref Y as single, byref Z as single )

' get the absolute position of the node in the scene rather than its relative
' position in relation to its parent
declare sub IrrGetNodeAbsolutePosition CDECL alias "IrrGetNodeAbsolutePosition" ( byval node as irr_node, byref X as single, byref Y as single, byref Z as single )

' get the rotation of the node in the scene
declare sub IrrGetNodeRotation CDECL alias "IrrGetNodeRotation" ( byval node as irr_node, byref X as single, byref Y as single, byref Z as single )


' set the portion of the animation sequence in the node that is to be played
declare sub IrrSetNodeAnimationRange CDECL alias "IrrSetNodeAnimationRange" ( byval node as irr_node, byval startFrame as integer, byval iEndFrame as integer )

' set the MD2 animation sequence to playback in the animation
declare sub IrrPlayNodeMD2Animation CDECL alias "IrrPlayNodeMD2Animation" ( byval node as irr_node, byval sequence as uinteger )

' set the speed of animation playback
declare sub IrrSetNodeAnimationSpeed CDECL alias "IrrSetNodeAnimationSpeed" ( byval node as irr_node, byval speed as integer )

' get the current frame number being played in the animation
declare function IrrGetNodeAnimationFrame CDECL alias "IrrGetNodeAnimationFrame" ( byval node as irr_node ) as uinteger

' set the current frame number being played in the animation
declare sub IrrSetNodeAnimationFrame CDECL alias "IrrSetNodeAnimationFrame" ( byval node as irr_node, byval frame as integer )

' set the time in seconds across which two animation frames are blended
declare sub IrrSetTransitionTime CDECL alias "IrrSetTransitionTime" ( byval node as irr_node, byval speed as single )

'Animates the mesh based on the position of the joints, this should be used at
'the end of any manual joint operations including blending and joints animated
'using IRR_JOINT_MODE_CONTROL and IrrSetNodeRotation on a bone node
declare sub IrrAnimateJoints CDECL alias "IrrAnimateJoints" ( byval node as irr_node )

' Sets the animation mode of joints in a node
' IRR_JOINT_MODE_NONE will result in no animation of the model based on bones
' IRR_JOINT_MODE_READ will result in automatic animation based upon 
' IRR_JOINT_MODE_CONTROL will allow the position of the bones to be set through code
' When using the control mode IrrAnimateJoints must be called before IrrDrawScene
declare sub IrrSetJointMode CDECL alias "IrrSetJointMode" ( byval node as irr_node, byval mode as uinteger )


' get a node that represents the position of a joint in a Milkshape skeleton
declare function IrrGetJointNode CDECL alias "IrrGetJointNode" ( byval node as irr_node, byval joint_name as zstring ptr ) as irr_node

' get a node that represents the position of a joint in a Milkshape skeleton
' (note this function is now redundant please use IrrGetJointNode instead)
declare function IrrGetMS3DJointNode CDECL alias "IrrGetMS3DJointNode" ( byval node as irr_node, byval joint_name as zstring ptr ) as irr_node

' get a node that represents the position of a joint in a Direct X skeleton
' (note this function is now redundant please use IrrGetJointNode instead)
declare function IrrGetDirectXJointNode CDECL alias "IrrGetDirectXJointNode" ( byval node as irr_node, byval joint_name as zstring ptr ) as irr_node

' adds a node to another node as a child to a parent
declare sub IrrAddChildToParent CDECL alias "IrrAddChildToParent" ( byval child as irr_node, byval parent as irr_node )


' add a shadow to a node
declare sub IrrAddNodeShadow CDECL alias "IrrAddNodeShadow" ( byval node as irr_node )

' set whether a node is visible or not
declare sub IrrSetNodeVisibility CDECL alias "IrrSetNodeVisibility" ( byval node as irr_node, byval visible as integer )

' delete this node from the scene
declare sub IrrRemoveNode CDECL alias "IrrRemoveNode" ( byval node as irr_node )

' Clears the entire scene, any references to nodes in the scene will become invalid
declare sub IrrRemoveAllNodes CDECL alias "IrrRemoveAllNodes" ()


' get the parent of this node, returns 0 if there is no parent
declare function IrrGetNodeParent CDECL alias "IrrGetNodeParent" ( byval node as irr_node ) as irr_node

' set the parent of this node
declare sub IrrSetNodeParent CDECL alias "IrrSetNodeParent" ( byval node as irr_node, byval parent as irr_node )

' get the first child node of this node, returns 0 if there is no child
declare function IrrGetNodeFirstChild CDECL alias "IrrGetNodeFirstChild" ( _
        byval node as irr_node, byref position as any ptr ) as irr_node

' get the next child node of this node, returns 0 if there is no child
declare function IrrGetNodeNextChild CDECL alias "IrrGetNodeNextChild" ( _
        byval node as irr_node, byref position as any ptr ) as irr_node

' returns true if this is the last child of the parent
declare function IrrIsNodeLastChild CDECL alias "IrrIsNodeLastChild" ( _
        byval node as irr_node, byref position as any ptr ) as irr_node

' get the ID of this node
declare function IrrGetNodeID CDECL alias "IrrGetNodeID" ( byval node as irr_node ) as integer

' set the ID of this node
declare sub IrrSetNodeID CDECL alias "IrrSetNodeID" ( byval node as irr_node, byval id as integer )

' get the name of this node
declare function IrrGetNodeName CDECL alias "IrrGetNodeName" ( byval node as irr_node ) as const zstring ptr

' set the name of this node
declare sub IrrSetNodeName CDECL alias "IrrSetNodeName" ( byval node as irr_node, byval name as zstring ptr )



'' ////////////////////////////////////////////////////////////////////////////
'' Animator Functions

' add a collision animator to a node
declare function IrrAddCollisionAnimator CDECL alias "IrrAddCollisionAnimator" ( byval selector as irr_selector, byval node as irr_node, byval radiusx as single, byval radiusy as single, byval radiusz as single, byval gravityx as single, byval gravityy as single, byval gravityz as single, byval offsetx as single, byval offsety as single, byval offsetz as single ) as irr_animator

' add a deletion animator to a node
declare function IrrAddDeleteAnimator CDECL alias "IrrAddDeleteAnimator" ( byval node as irr_node, byval delete_after_n_milliseconds as integer ) as irr_animator

' add a fly-in-circle animator to a node
declare function IrrAddFlyCircleAnimator CDECL alias "IrrAddFlyCircleAnimator" ( byval node as irr_node, byval centre_x as single, byval centre_y as single, byval centre_z as single, byval radius as single, byval speed as single ) as irr_animator

' add a fly-straight animator to a node
declare function IrrAddFlyStraightAnimator CDECL alias "IrrAddFlyStraightAnimator" ( byval node as irr_node, byval start_x as single, byval start_y as single, byval start_z as single, byval end_x as single, byval end_y as single, byval end_z as single, byval time_to_complete as uinteger, byval loop_path as integer ) as irr_animator

' add a rotation animator to a node
declare function IrrAddRotationAnimator CDECL alias "IrrAddRotationAnimator" ( byval node as irr_node, byval x as single, byval y as single, byval z as single )as irr_animator

' add a spline animator to a node
declare function IrrAddSplineAnimator CDECL alias "IrrAddSplineAnimator" ( byval node as irr_node, byval array_size as integer, byref x as single, byref y as single, byref z as single, byval time_to_start as integer, byval speed as single, byval tightness as single ) as irr_animator

' remove an animator from a node
declare sub IrrRemoveAnimator CDECL alias "IrrRemoveAnimator" ( byval node as irr_node, byval node as irr_animator )



'' ////////////////////////////////////////////////////////////////////////////
'' Collision Functions

' gets a collision object from a complex mesh like a map
declare function IrrGetCollisionGroupFromComplexMesh CDECL alias "IrrGetCollisionGroupFromComplexMesh" (byval mesh as irr_mesh, byval node as irr_node) as irr_selector

' creates a collision object from triangles in a node defined by its
' bounding box
declare function IrrGetCollisionGroupFromBox CDECL alias "IrrGetCollisionGroupFromBox" ( byval node as irr_node ) as irr_selector

' creates a collision selector from a terrain node
declare function IrrGetCollisionGroupFromTerrain CDECL alias "IrrGetCollisionGroupFromTerrain" ( byval node as irr_node, byval level_of_detail as integer ) as irr_selector

' creates a combined collision object that is a group of collision objects
' for example you could combine a map and a terrain
declare function IrrCreateCombinedCollisionGroup CDECL alias "IrrCreateCombinedCollisionGroup" () as irr_selector

' adds a collision object to a combined collision object
declare sub IrrAddCollisionGroupToCombination CDECL alias "IrrAddCollisionGroupToCombination" ( byval combined_collision_group as irr_selector, byval collision_group as irr_selector )

' remove all collision objects from the combined collision object
declare sub IrrRemoveAllCollisionGroupsFromCombination CDECL alias "IrrRemoveAllCollisionGroupsFromCombination" ( byval combined_collision_group as irr_selector )

' remove a particular collision object from the combined collision object
declare sub IrrRemoveCollisionGroupFromCombination CDECL alias "IrrRemoveCollisionGroupFromCombination" ( byval combined_collision_group as irr_selector, byval collision_group as irr_selector )

' detect the collision point of a ray in the scene with a collision object if a
' collision was detected 1 is returned and vector collision contains the
' co-ordinates of the point of collision
declare function IrrGetCollisionPoint CDECL alias "IrrGetCollisionPoint" ( byref start as IRR_VECTOR, byref line_end as IRR_VECTOR, byval collision_group as irr_selector, byref collision_point as IRR_VECTOR ) as integer

' get a ray that goes from the specified camera and through the screen
' coordinates the information is coppied into the supplied start and end vectors
declare sub IrrGetRayFromScreenCoordinates CDECL alias "IrrGetRayFromScreenCoordinates" ( byval screen_x as integer, byval screen_y as integer, byval camera as irr_camera, byref start as IRR_VECTOR, byref line_end as IRR_VECTOR )

' a ray is cast through the camera and the nearest node that is hit by the ray
' is returned. if no node is hit zero is returned for the object
declare function IrrGetCollisionNodeFromCamera CDECL alias "IrrGetCollisionNodeFromCamera" ( byval camera as irr_camera ) as irr_node

' a ray is cast through the supplied coordinates and the nearest node that is
' hit by the ray is returned. if no node is hit zero is returned for the object
declare function IrrGetCollisionNodeFromRay CDECL alias "IrrGetCollisionNodeFromRay" ( byref start as IRR_VECTOR, byref line_end as IRR_VECTOR ) as irr_node

' a ray is cast through the supplied coordinates and the nearest node that is hit
' by the ray is returned. if no node is hit zero is returned for the object, only
' a subset of objects are tested, i.e. the children of the supplied node that
' match the supplied id. if the recurse option is enabled the entire tree of child
' objects connected to this node are tested
declare function IrrGetChildCollisionNodeFromRay CDECL alias "IrrGetChildCollisionNodeFromRay" ( _
        byval node as irr_node, _
        byval idMask as integer, _
        byval recurse as uinteger, _
        byref start as IRR_VECTOR, _
        byref line_end as IRR_VECTOR ) as irr_node

' the node and its children are recursively tested and the first node that contains
' the matched point is returned. if no node is hit zero is returned for the object,
' only a subset of objects are tested, i.e. the children of the supplied node that
' match the supplied id. if the recurse option is enabled the entire tree of child
' objects connected to this node are tested
declare function IrrGetChildCollisionNodeFromPoint CDECL alias "IrrGetChildCollisionNodeFromPoint" ( _
        byval node as irr_node, _
        byval idMask as integer, _
        byval recurse as uinteger, _
        byref point as IRR_VECTOR ) as irr_node

' a ray is cast through the screen at the specified co-ordinates and the nearest
' node that is hit by the ray is returned. if no node is hit zero is returned
' for the object
declare function IrrGetCollisionNodeFromScreenCoordinates CDECL alias "IrrGetCollisionNodeFromScreenCoordinates" ( byval screen_x as integer, byval screen_y as integer ) as irr_node

' screen co-ordinates are returned for the position of the specified 3d
' co-ordinates if an object were drawn at them on the screen, this is ideal for
' drawing 2D bitmaps or text around or on your 3D object on the screen
declare sub IrrGetScreenCoordinatesFrom3DPosition CDECL alias "IrrGetScreenCoordinatesFrom3DPosition" ( byref screen_x as integer, byref screen_y as integer, byval at_position as IRR_VECTOR )

' an all-inclusive collision routine for .irr scenes
Declare Sub IrrSetupIrrSceneCollision CDECL Alias "IrrSetupIrrSceneCollision" ( ByVal camera As irr_camera ) 



'' ////////////////////////////////////////////////////////////////////////////
'' Camera Functions

' add a configurable first person perspective camera to the scene
Declare Function IrrAddFPSCamera CDECL Alias "IrrAddFPSCamera" ( ByVal parent As irr_scene_node = 0, ByVal rotateSpeed As Single = 100.0, ByVal moveSpeed As Single = 500.0, ByVal id As Integer = -1, ByVal keyMapArray As SKeyMap Ptr = 0, ByVal keyMapSize As Integer = 0, ByVal noVericalMovement As Byte = 0, ByVal jumpSpeed As Single = 0.0) As irr_camera

' add a regular camera
declare function IrrAddCamera CDECL alias "IrrAddCamera" ( byval cameraX as single, byval cameraY as single, byval cameraZ as single, byval targetX as single, byval targetY as single, byval targetZ as single ) as irr_camera

' add a maya style camera to the scene
declare function IrrAddMayaCamera CDECL alias "IrrAddMayaCamera" ( ByVal parent As irr_scene_node = 0, ByVal rotateSpeed As Single = 1500.0, ByVal zoomSpeed As Single = 200.0, ByVal moveSpeed as Single = 1500.0) as irr_camera
'declare function IrrAddMayaCamera CDECL alias "IrrAddMayaCamera" ( byval parent as irr_scene


' reposition the target location of a camera
declare sub IrrSetCameraTarget CDECL alias "IrrSetCameraTarget" ( byval camera as irr_camera, byval X as single, byval Y as single, byval Z as single )

' get the current target location of a camera, the location is copied into the
' supplied variables
declare sub IrrGetCameraTarget CDECL alias "IrrGetCameraTarget" ( byval camera as irr_camera, byref X as single, byref Y as single, byref Z as single )

' set the up vector of a camera object, this controls the upward direction of
' the camera and allows you to roll it for free flight action
declare sub IrrSetCameraUpDirection CDECL alias "IrrSetCameraUpDirection" ( byval camera as irr_camera, byval X as single, byval Y as single, byval Z as single )

' get the vectors describing the camera direction useful after the camera has been revolved
declare sub IrrGetCameraOrientation CDECL alias "IrrGetCameraOrientation" ( _
        byval camera as irr_camera, _
        byref X as IRR_VECTOR, _
        byref Y as IRR_VECTOR, _
        byref Z as IRR_VECTOR )

' set the distance at which the camera starts to clip polys
declare sub IrrSetCameraClipDistance CDECL alias "IrrSetCameraClipDistance" ( byval camera as irr_camera, byval distance as single )

' set the active camera in the scene
declare sub IrrSetActiveCamera CDECL alias "IrrSetActiveCamera" ( byval camera as irr_camera )

'sets the field of view (Default: PI / 2.5f). 
declare sub IrrSetCameraFOV CDECL alias "IrrSetCameraFOV" ( byval camera as irr_camera, byval fov as single )

'revolve the camera using quaternion calculations, this will help avoid gimbal
'lock associated with normal Rotations many thanks to RogerBorg for this
declare sub IrrRevolveCamera CDECL alias "IrrRevolveCamera" ( _
        byval camera as irr_camera, _
        byval yaw as single, _
        byval pitch as single, _
        byval roll as single, _
        byval drive as single, _
        byval strafe as single, _
        byval elevate as single )

declare sub IrrSetCameraUpAtRightAngle CDECL alias "IrrSetCameraUpAtRightAngle" ( byval camera as irr_camera )


'' ////////////////////////////////////////////////////////////////////////////
'' Lighting Functions

'add a light to the  scene
Declare Function IrrAddLight CDECL Alias "IrrAddLight" ( ByVal parent As irr_node = 0, ByVal x As Single, ByVal y As Single, ByVal z As Single, ByVal red As Single, ByVal green As Single, ByVal blue As Single, ByVal size As Single ) As irr_node

' set the scene ambient lighting
declare sub IrrSetAmbientLight CDECL alias "IrrSetAmbientLight" ( byval R as single, byval G as single, byval B as single )


' Ambient color emitted by the light.
declare sub IrrSetLightAmbientColor CDECL alias "IrrSetLightAmbientColor" ( byval light as irr_node, byval R as single, byval G as single, byval B as single )

' Changes the light strength fading over distance. Good values for distance
' effects use ( 1.0, 0.0, 0.0) and simply add small values to the second and
' third element.
declare sub IrrSetLightAttenuation CDECL alias "IrrSetLightAttenuation" ( byval light as irr_node, byval constant as single, byval linear as single, byval quadratic as single )

' Does the light cast shadows?
declare sub IrrSetLightCastShadows CDECL alias "IrrSetLightCastShadows" ( byval light as irr_node, byval cast_shadows as uinteger )

' Diffuse color emitted by the light.
declare sub IrrSetLightDiffuseColor CDECL alias "IrrSetLightDiffuseColor" ( byval light as irr_node, byval R as single, byval G as single, byval B as single )

' The light strength's decrease between Outer and Inner cone.
declare sub IrrSetLightFalloff CDECL alias "IrrSetLightFalloff" ( byval light as irr_node, byval Falloff as single )

' The angle of the spot's inner cone. Ignored for other lights.
declare sub IrrSetLightInnerCone CDECL alias "IrrSetLightInnerCone" ( byval light as irr_node, byval InnerCone as single )

' The angle of the spot's outer cone. Ignored for other lights.
declare sub IrrSetLightOuterCone CDECL alias "IrrSetLightOuterCone" ( byval light as irr_node, byval OuterCone as single )

' Radius of light. Everything within this radius be be lighted.
declare sub IrrSetLightRadius CDECL alias "IrrSetLightRadius" ( byval light as irr_node, byval Radius as single )

' Specular color emitted by the light.
declare sub IrrSetLightSpecularColor CDECL alias "IrrSetLightSpecularColor" ( byval light as irr_node, byval R as single, byval G as single, byval B as single )

' Type of the light. Default: ELT_POINT.
declare sub IrrSetLightType CDECL alias "IrrSetLightType" ( byval light as irr_node, byval light_type as E_LIGHT_TYPE )



'' ////////////////////////////////////////////////////////////////////////////
'' Terrain Functions

' create a terrain node from a highfield map
'declare function IrrAddTerrain CDECL alias "IrrAddTerrain" ( byval path as zstring ptr ) as irr_terrain

' create a terrain node from a highfield map
declare function IrrAddSphericalTerrain CDECL alias "IrrAddSphericalTerrain" ( _
        byval topPath as zstring ptr, _
        byval frontPath as zstring ptr, _
        byval backPath as zstring ptr, _
        byval leftPath as zstring ptr, _
        byval rightPath as zstring ptr, _
        byval bottomPath as zstring ptr, _
        byval xPosition as single = 0.0, _
        byval yPosition as single = 0.0, _
        byval zPosition as single = 0.0, _
        byval xRotation as single = 0.0, _
        byval yRotation as single = 0.0, _
        byval zRotation as single = 0.0, _
        byval xScale as single = 1.0, _
        byval yScale as single = 1.0, _
        byval zScale as single = 1.0, _
        byval vertexAlpha as integer = 255, _
        byval vertexRed as integer = 255, _
        byval vertexGreen as integer = 255, _
        byval vertexBlue as integer = 255, _
        byval smoothing as integer = 0, _
        byval spherical as integer = 0, _
        byval maxLOD as integer = 5, _
        byval patchSize as IRR_TERRAIN_PATCH_SIZE = ETPS_17 ) as irr_terrain

' create a tiled terrain node from a highfield map
declare function IrrAddTiledTerrain CDECL alias "IrrAddTiledTerrain" ( _
        byval heightMapPath as zstring ptr, _
        byval texturePath as zstring ptr, _
        byval followNode as irr_node, _
        byval terrainWidth as integer = 1000, _
        byval terrainHeight as integer = 1000, _
        byval tileSize as single = 1.0, _
        byval meshSize as integer = 100, _
        byval heightmapScale as single = 10.0, _
        byval smoothing as integer = 0.0, _
        byval materialType as IRR_MATERIAL_TYPES = IRR_EMT_DETAIL_MAP ) as irr_node

' create a terrain node from a highfield map
declare function IrrAddTerrain CDECL alias "IrrAddTerrain" ( _
        byval path as zstring ptr, _
        byval xPosition as single = 0.0, _
        byval yPosition as single = 0.0, _
        byval zPosition as single = 0.0, _
        byval xRotation as single = 0.0, _
        byval yRotation as single = 0.0, _
        byval zRotation as single = 0.0, _
        byval xScale as single = 1.0, _
        byval yScale as single = 1.0, _
        byval zScale as single = 1.0, _
        byval vertexAlpha as integer = 255, _
        byval vertexRed as integer = 255, _
        byval vertexGreen as integer = 255, _
        byval vertexBlue as integer = 255, _
        byval smoothing as integer = 0, _
        byval maxLOD as integer = 5, _
        byval patchSize as IRR_TERRAIN_PATCH_SIZE = ETPS_17 ) as irr_terrain


' scale the texture on a terrain node
declare sub IrrScaleTexture CDECL alias "IrrScaleTexture" ( byval terrain as irr_terrain, byval X as single, byval Y as single )

' set the textures on all six faces of a spherical terrain
declare sub IrrSetSphericalTerrainTexture CDECL alias "IrrSetSphericalTerrainTexture" ( _
        byval terrain as irr_terrain, _
        byval topTexture as irr_texture, _
        byval frontTexture as irr_texture, _
        byval backTexture as irr_texture, _
        byval leftTexture as irr_texture, _
        byval rightTexture as irr_texture, _
        byval bottomTexture as irr_texture, _
        byval materialIndex as uinteger )
        
' load the vertex colors on all six faces of a spherical terrain from images
declare sub IrrLoadSphericalTerrainVertexColor CDECL alias "IrrLoadSphericalTerrainVertexColor" ( _
        byval terrain as irr_terrain, _
        byval topImage as irr_image, _
        byval frontImage as irr_image, _
        byval backImage as irr_image, _
        byval leftImage as irr_image, _
        byval rightImage as irr_image, _
        byval bottomImage as irr_image )

' scale the texture on a spherical terrain node
declare sub IrrScaleSphericalTexture CDECL alias "IrrScaleSphericalTexture" ( byval terrain as irr_terrain, byval X as single, byval Y as single )

' get the height of a co-ordinate on a tiled terrain node
declare function IrrGetTerrainHeight CDECL alias "IrrGetTerrainHeight" ( byval terrain as irr_terrain, byval X as single, byval Z as single ) as single

' get the height of a co-ordinate on a tiled terrain node
declare function IrrGetTiledTerrainHeight CDECL alias "IrrGetTiledTerrainHeight" ( byval terrain as irr_terrain, byval X as single, byval Z as single ) as single

' get the surface position of a co-ordinate on a spherical terrain node
declare sub IrrGetSphericalTerrainSurfacePosition CDECL alias "IrrGetSphericalTerrainSurfacePosition" ( _
        byval terrain as irr_terrain, _
        byval face as integer, _
        byval logicalX as single, _
        byval logicalZ as single, _
        byref X as single, _
        byref Y as single, _
        byref Z as single )

' get the surface angle of a co-ordinate on a spherical terrain node
declare sub IrrGetSphericalTerrainSurfacePositionAndAngle CDECL alias "IrrGetSphericalTerrainSurfacePositionAndAngle" ( _
        byval terrain as irr_terrain, _
        byval face as integer, _
        byval logicalX as single, _
        byval logicalZ as single, _
        byref positionX as single, _
        byref positionY as single, _
        byref positionZ as single, _
        byref angleX as single, _
        byref angleY as single, _
        byref angleZ as single )

' get the logical position on a surface based on a real co-ordinate
declare sub IrrGetSphericalTerrainLogicalSurfacePosition CDECL alias "IrrGetSphericalTerrainLogicalSurfacePosition" ( _
        byval terrain as irr_terrain, _
        byval X as single, _
        byval Y as single, _
        byval Z as single, _
        byref face as integer, _
        byref logicalX as single, _
        byref logicalZ as single )

' create a terrain node from a highfield map
declare function IrrAddTerrainTile CDECL alias "IrrAddTerrainTile" ( _
        byval image as irr_image, _
        byval tileSize as integer = 256, _
        byval dataX as integer = 0, _
        byval dataY as integer = 0, _
        byval xPosition as single = 0.0, _
        byval yPosition as single = 0.0, _
        byval zPosition as single = 0.0, _
        byval xRotation as single = 0.0, _
        byval yRotation as single = 0.0, _
        byval zRotation as single = 0.0, _
        byval xScale as single = 1.0, _
        byval yScale as single = 1.0, _
        byval zScale as single = 1.0, _
        byval smoothing as integer = 1, _
        byval maxLOD as integer = 5, _
        byval patchSize as IRR_TERRAIN_PATCH_SIZE = ETPS_17 ) as irr_terrain

' get the height of a point on a terrain tile node
declare function IrrGetTerrainTileHeight CDECL alias "IrrGetTerrainTileHeight" ( byval terrain as irr_terrain, byval X as single, byval Z as single ) as single

' scale the texture detail on a tiled terrain node
declare sub IrrScaleTileTexture CDECL alias "IrrScaleTileTexture"  ( byval terrain as irr_terrain, byval X as single, byval Y as single )

' attach a tile onto this tile
#define TOP_EDGE 0
#define BOTTOM_EDGE 1
#define LEFT_EDGE 2
#define RIGHT_EDGE 3
declare sub IrrAttachTile CDECL alias "IrrAttachTile"  ( byval terrain as irr_terrain, byval neighbour as irr_terrain, byval Edge as uinteger )

' load the vertex color information of a tile
declare sub IrrSetTileColor CDECL alias "IrrSetTileColor"  ( _
        byval terrain as irr_terrain, _
        byval image as irr_image, _
        byval dataX as integer = 0, _
        byval dataY as integer = 0 )

' load the height and UV data from an image
declare sub IrrSetTileStructure CDECL alias "IrrSetTileStructure"  ( _
        byval terrain as irr_terrain, _
        byval image as irr_image, _
        byval dataX as integer = 0, _
        byval dataY as integer = 0 )



'' ////////////////////////////////////////////////////////////////////////////
'' Particle Functions

' set the size of a particle in the particle system
declare sub IrrSetParticleSize CDECL alias "IrrSetParticleSize" ( byval particle_system as irr_particle_system, byval X as single, byval Y as single )

' create an emitter that can be added to a particle system
declare function IrrAddParticleEmitter CDECL alias "IrrAddParticleEmitter" ( byval particle_system as irr_particle_system, byval settings as IRR_PARTICLE_EMITTER ) as irr_emitter

' Creates a particle emitter for an animated mesh scene node
declare function IrrAddAnimatedMeshSceneNodeEmitter CDECL alias "IrrAddAnimatedMeshSceneNodeEmitter" ( byval particle_system as irr_particle_system, byval node as irr_node, byval use_normal_direction as uinteger, byval normal_direction_modifier as single, byval emit_from_every_vertex as integer, byval settings as IRR_PARTICLE_EMITTER ) as irr_emitter


' Add an affector to the particle system to fade the particles out
declare function IrrAddFadeOutParticleAffector CDECL alias "IrrAddFadeOutParticleAffector" ( byval particle_system as irr_particle_system, byval fade_speed as uinteger, byval fade_to_red as uinteger, byval fade_to_green as uinteger, byval fade_to_blue as uinteger ) as irr_affector

' Add an affector to the particle system to alter their position with gravity
declare function IrrAddGravityParticleAffector CDECL Alias "IrrAddGravityParticleAffector" ( ByVal particle_system As irr_particle_system, ByVal x As Single, ByVal y As Single, ByVal z As Single, ByVal timeForceLost As uInteger = 1000 ) as irr_affector

' Creates a point attraction affector. This affector modifies the positions of
' the particles and attracts them to a specified point at a specified speed per
' second.
declare function IrrAddParticleAttractionAffector CDECL Alias "IrrAddParticleAttractionAffector" ( ByVal particle_system As irr_particle_system, ByVal x As Single, ByVal y As Single, ByVal z As Single, ByVal speed As Single = 1.0, ByVal attract as uinteger = 1, ByVal affectX as uinteger = 1, ByVal affectY as uinteger = 1, ByVal affectZ as uinteger = 1 ) as irr_affector

' Creates a rotation affector. This affector modifies the positions of the
' particles and attracts them to a specified point at a specified speed per second.
declare function IrrAddRotationAffector CDECL Alias "IrrAddRotationAffector" ( ByVal particle_system As irr_particle_system, ByVal Speed_X As Single, ByVal Speed_Y As Single, ByVal Speed_Z As Single, ByVal pivot_X As Single, ByVal pivot_Y As Single, ByVal pivot_Z As Single ) as irr_affector

' Creates a Stop Particle Affector that automatically stops the emmision of
' particles from the specified emitter after a period of time
declare function IrrAddStopParticleAffector CDECL Alias "IrrAddStopParticleAffector" ( ByVal particle_system As irr_particle_system, ByVal run_time as uinteger, ByVal target_emitter as irr_emitter ) as irr_affector

' adds a push affector with a limited range of effect
declare function IrrAddParticlePushAffector CDECL Alias "IrrAddParticlePushAffector" ( _
		ByVal particle_system as irr_particle_system, _
        ByVal x as single, _
		ByVal y as single, _
		ByVal z as single, _
		ByVal speedX as single, _
		ByVal speedY as single, _
		ByVal speedZ as single, _
		ByVal far as single, _
		ByVal near as single, _
        ByVal column as single, _
		ByVal radial as integer ) as irr_affector

' adds a color affector to change the color of particles over time through an
' array of user defined colors
declare function IrrAddColorMorphAffector CDECL Alias "IrrAddColorMorphAffector" ( _
		ByVal particle_system as irr_particle_system, _
        ByVal num as uinteger, _
		ByVal particlecolors as uinteger ptr, _
		ByVal particletimes as uinteger ptr, _
		ByVal smooth as uinteger ) as irr_affector

' adds a spline affector to move particles along the path of a user defined
' spline
declare function IrrAddSplineAffector CDECL Alias "IrrAddSplineAffector" ( _
		ByVal particle_system as irr_particle_system, _
        ByVal VertexCount as uinteger, _
		ByVal verticies as IRR_VERT ptr, _
		ByVal speed as single, _
        ByVal tightness as single, _
        ByVal attraction as single, _
		ByVal deleteAtEnd as uinteger ) as irr_affector

' remove all effectors from this particle system
declare sub IrrRemoveAffectors CDECL alias "IrrRemoveAffectors" ( byval particle_system as irr_particle_system )


' Set direction the emitter emits particles.
declare sub IrrSetParticleEmitterDirection CDECL alias "IrrSetParticleEmitterDirection" ( byval particle_system As irr_emitter, byval x as single, byval y as single, byval z as single )

' Set minimum number of particles the emitter emits per second.
declare sub IrrSetParticleEmitterMinParticlesPerSecond CDECL alias "IrrSetParticleEmitterMinParticlesPerSecond" ( byval particle_system As irr_emitter, byval minPPS as uinteger )

' Set maximum number of particles the emitter emits per second.
declare sub IrrSetParticleEmitterMaxParticlesPerSecond CDECL alias "IrrSetParticleEmitterMaxParticlesPerSecond" ( byval particle_system As irr_emitter, byval maxPPS as uinteger )

' Set minimum starting color for particles.
declare sub IrrSetParticleEmitterMinStartColor CDECL alias "IrrSetParticleEmitterMinStartColor" ( byval particle_system As irr_emitter, byval Red as uinteger, byval Green as uinteger, byval Blue as uinteger )

' Set maximum starting color for particles.
declare sub IrrSetParticleEmitterMaxStartColor CDECL alias "IrrSetParticleEmitterMaxStartColor" ( byval particle_system As irr_emitter, byval Red as uinteger, byval Green as uinteger, byval Blue as uinteger )


' Enable or disable an affector
declare sub IrrSetParticleAffectorEnable CDECL alias "IrrSetParticleAffectorEnable" ( byval affector as irr_affector, byval enable as uinteger )

' Alter the fadeout affector changing the fade out time
declare sub IrrSetFadeOutParticleAffectorTime CDECL alias "IrrSetFadeOutParticleAffectorTime" ( byval affector as irr_affector, byval fade_speed as single )

' Alter the fadeout affector changing the target color
declare sub IrrSetFadeOutParticleAffectorTargetColor CDECL alias "IrrSetFadeOutParticleAffectorTargetColor" ( byval affector as irr_affector, byval fade_to_red as uinteger, byval fade_to_green as uinteger, byval fade_to_blue as uinteger )

' Alter the direction and force of gravity
declare sub IrrSetGravityParticleAffectorDirection CDECL alias "IrrSetGravityParticleAffectorDirection" ( byval affector as irr_affector, byval x as single, byval y as single, byval z as single )

' Set the time in milliseconds when the gravity force is totally lost and the particle does not move any more
declare sub IrrSetGravityParticleAffectorTimeForceLost CDECL alias "IrrSetGravityParticleAffectorTimeForceLost" ( byval affector as irr_affector, byval time_Force_Lost as single )

' Set whether or not this will affect particles in the X direction.
declare sub IrrSetParticleAttractionAffectorAffectX CDECL alias "IrrSetParticleAttractionAffectorAffectX" ( byval affector as irr_affector, byval affect_x as uinteger )

' Set whether or not this will affect particles in the Y direction.
declare sub IrrSetParticleAttractionAffectorAffectY CDECL alias "IrrSetParticleAttractionAffectorAffectY" ( byval affector as irr_affector, byval affect_y as uinteger )

' Set whether or not this will affect particles in the Z direction.
declare sub IrrSetParticleAttractionAffectorAffectZ CDECL alias "IrrSetParticleAttractionAffectorAffectZ" ( byval affector as irr_affector, byval affect_z as uinteger )

' Set whether or not the particles are attracting or detracting.
declare sub IrrSetParticleAttractionAffectorAttract CDECL alias "IrrSetParticleAttractionAffectorAttract" ( byval affector as irr_affector, byval attract as uinteger )
 
' Set the point that particles will attract to. 
declare sub IrrSetParticleAttractionAffectorPoint CDECL alias "IrrSetParticleAttractionAffectorPoint" ( byval affector as irr_affector, byval x as single, byval y as single, byval z as single )

' Set the point that particles will rotate about.
declare sub IrrSetRotationAffectorPivotPoint CDECL alias "IrrSetRotationAffectorPivotPoint" ( byval affector as irr_affector, byval x as single, byval y as single, byval z as single )

' Set the furthest distance for the push particle effect
declare sub IrrSetFurthestDistanceOfEffect CDECL alias "IrrSetFurthestDistanceOfEffect" ( byval affector as irr_affector, byval NewDistance as single )

' Set the nearest distance for the push particle effect
declare sub IrrSetNearestDistanceOfEffect CDECL alias "IrrSetNearestDistanceOfEffect" ( byval affector as irr_affector, byval NewDistance as single )

' Set the column width of the push particle effect
declare sub IrrSetColumnDistanceOfEffect CDECL alias "IrrSetColumnDistanceOfEffect" ( byval affector as irr_affector, byval NewDistance as single )

' Set the center of the push particle effect
declare sub IrrSetCenterOfEffect CDECL alias "IrrSetCenterOfEffect" ( byval affector as irr_affector, byval x as single, byval y as single, byval z as single )

' Set the strength of the push particle effect
declare sub IrrSetStrengthOfEffect CDECL alias "IrrSetStrengthOfEffect" ( byval affector as irr_affector, byval x as single, byval y as single, byval z as single )

' Set the radial effect of the push particle effect
declare sub IrrSetRadialEffect CDECL alias "IrrSetRadialEffect" ( byval affector as irr_affector, byval IsRadial as integer )



'' ////////////////////////////////////////////////////////////////////////////
'' GUI Functions

' add a static text object to the gui display
declare sub IrrAddStaticText CDECL alias "IrrAddStaticText" ( byval text as wstring ptr, byval TopX as integer, byval TopY as integer, byval BotX as integer, byval BotY as integer, byval border as uinteger, byval wordwrap as uinteger )
